% [steeringAngle, stop, laplot, pathDist] =...
%     followTheCarrot(obstacleAvoidance, handles, simMode)
% 
% Makes a vehicle follow a predefined path
%
% obstacleAvoidance: 'vfh' or 'vfhPlus' 
% handles: handles to the GUI components
% simMode: 'sim' if simultion, 'mod' if model, 'real' if real vehicle
%
% Returns:
% steeringAngle: the selected steering angle in radians
% stop: 1 if the vehicle encounteres problems and must stop, else 0
% laPlot: handle to the look ahead point plot
% pathDist: distance between the path and the vehicle
%
function [steeringAngle, stop, laplot, pathDist] =...
    followTheCarrot(obstacleAvoidance, handles, simMode)

global thePath vehicle vehicleSize
nextWayPoint = 0; stop=0;
kp = 1; % propulsion gain
currPos = [vehicle.x vehicle.y]; % real position
believedPos = currPos + vehicle.rand; % simulate noice in the position
PTH = [thePath.x thePath.y];
lookAhead =3.5*vehicle.length; % look ahead distance (== 11.9131  ~5.1m in front of the vehicle))
%lookAhead =5.5*vehicle.length % look ahead distance (== 18.7206  ~11.9m in front of the vehicle))
% VCP -> front == 6.8075
nrOfPoints = length(PTH);
[row, pathPoint, pathDist] = getPathPoint(believedPos, PTH); % find nearest point on the path
segmentVector = PTH(row,:) - PTH(row+1,:);
[pathAngle, r] = cart2pol(segmentVector(1), segmentVector(2));
lookAheadPoint = pathPoint - [cos(pathAngle)*lookAhead, sin(pathAngle)*lookAhead];

[rw,pp,d] = getPathPoint(lookAheadPoint, PTH);
[d,s,p] = pointLineDist(lookAheadPoint,PTH(row,:), PTH(row+1,:));
if(rw == nrOfPoints-1 & s == 0)
    lookAheadPoint = PTH(nrOfPoints,:); % set the last look ahead point to goal point
else
    j=row;
    while(j<nrOfPoints-1)
        SP = (PTH(j,:)); %startpoint
        EP = (PTH(j+1,:)); %endpoint
        [rest, onSegment, point] = pointLineDist(lookAheadPoint, SP, EP );	
        if(onSegment == 0 & rest > 0)
            segmentVector = PTH(j+1,:) - PTH(j+2,:);
            [pathAngle, r] = cart2pol(segmentVector(1), segmentVector(2));
            lookAheadPoint = PTH(j+1,:) - [cos(pathAngle)*rest, sin(pathAngle)*rest];
        else
            break
        end
        j=j+1;
    end
end
if(strcmp(obstacleAvoidance, 'VFH'))
    [lookAheadAngle, stop, noObstacles] = vfh(lookAheadPoint, handles, simMode);
elseif(strcmp(obstacleAvoidance, 'VFH+'))
    [lookAheadAngle, stop, noObstacles] = vfhPlus(lookAheadPoint, handles, simMode);
else
    lookAheadVector = lookAheadPoint - believedPos;
    [lookAheadAngle, D] = cart2pol(lookAheadVector(1), lookAheadVector(2));
end
errorAngle = (lookAheadAngle - vehicle.or) * kp;
if(errorAngle > pi)
    errorAngle = errorAngle - 2*pi;
elseif(errorAngle < -pi)
    errorAngle = errorAngle + 2*pi;
end
steeringAngle = max(min(errorAngle, deg2rad(40)),  deg2rad(-40));
steeringAngle = errorAngle;
laplot = plot(lookAheadPoint(1),lookAheadPoint(2), 'kX');

