% [steeringAngle, stop, laPlot, pathDist] =...
%       followThePast(obstacleAvoidance, handles, simMode)
%
% Makes a vehicle follow a predefined path
% 
% obstacleAvoidance: 'vfh' or 'vfhPlus' 
% handles: handles to the GUI components
% simMode: 'sim' if simultion, 'mod' if model, 'real' if real vehicle
%
% Returns:
% steeringAngle: the selected steering angle in radians
% stop: 1 if the vehicle encounteres problems and must stop, else 0
% laPlot: handle to the look ahead point plot
% pathDist: distance between the path and the vehicle
%
function [steeringAngle, stop, laPlot, pathDist] =...
    followThePast(obstacleAvoidance, handles, simMode)
global thePath vehicle
currPos = [vehicle.x vehicle.y]; % real position
believedPos = currPos + vehicle.rand; % simulate noice in the position

[row, pathPoint, pathDist] = getPathPoint(believedPos, [thePath.x thePath.y]); % find nearest point on the path
stop = 0;
nrOfPoints = length(thePath.x);
lookAhead =3.5*vehicle.length; % ~12m (~5.1m in front of the vehicle)
savedHead = thePath.or(row) + thePath.sa(row)*2; % saved heading
orientDiff = thePath.or(row) - vehicle.or; % difference between current and saved orientation
[lax,lay] = pol2cart(savedHead, lookAhead);
laPoint = pathPoint + [lax,lay]; %look ahead (carrot) point

[rw,pp,d] = getPathPoint(laPoint, [thePath.x thePath.y]);
[d,s,p] = pointLineDist(pp,[thePath.x(row) thePath.y(row)],[thePath.x(row+1) thePath.y(row+1)]);
if(rw >= nrOfPoints-1 & s == 0 & pathDist > 0.5)
    laPoint = [thePath.x(nrOfPoints), thePath.y(nrOfPoints)] ; % set the last look ahead point to goal point
end   
laVector = (laPoint - believedPos);
[laAngle, laDist] = cart2pol(laVector(1), laVector(2));
laAngle = mod(laAngle+2*pi, 2*pi);

if(strcmp(obstacleAvoidance, 'VFH'))
    [laAngle, stop, noObstacles] = vfh(laPoint, handles, simMode);
elseif(strcmp(obstacleAvoidance, 'VFH+'))
    [laAngle, stop, noObstacles] = vfhPlus(laPoint, handles, simMode);
end
errorAngle = laAngle-vehicle.or;
%  errorAngle = orientDiff + thePath.sa(row)*2; % Drive exactly like before (Test)
if(errorAngle > pi)
    errorAngle = errorAngle - 2*pi;
elseif(errorAngle < -pi)
    errorAngle = errorAngle + 2*pi;
end
% steeringAngle = max(min(errorAngle, deg2rad(40)),  deg2rad(-40));
steeringAngle = errorAngle;
laPlot = plot(laPoint(1),laPoint(2), 'kX');
