function varargout = gui(varargin)
% GUI Application M-file for gui.fig
%    FIG = GUI launch gui GUI.
%    GUI('callback_name', ...) invoke the named callback.

% Last Modified by GUIDE v2.0 28-Apr-2003 10:26:32

if nargin == 0  % LAUNCH GUI
    fig = openfig(mfilename,'reuse');
    % Use system color scheme for figure:
    set(fig,'Color',get(0,'defaultUicontrolBackgroundColor'));
    
    
    % Use system color scheme for figure:
    set(fig,'Color',get(0,'defaultUicontrolBackgroundColor'));
    
    % Generate a structure of handles to pass to callbacks, and store it. 
    handles = guihandles(fig);
    guidata(fig, handles);
    
    if nargout > 0
        varargout{1} = fig;
    end
    axes(handles.plotWin);
    xlabel('[m]')
    ylabel('[m]')
    %    btngroup(handles.plotWin);
    path_Callback(handles.path, [], handles, varargin); %draw the path
    obstacle_avoidance_Callback(handles.obstacle_avoidance, [], handles, varargin); %draw the obstacles
    
    
elseif ischar(varargin{1}) % INVOKE NAMED SUBFUNCTION OR CALLBACK
    try
        if (nargout)
            [varargout{1:nargout}] = feval(varargin{:}); % FEVAL switchyard
        else
            feval(varargin{:}); % FEVAL switchyard
        end
    catch
        disp(lasterr);
    end
    
end

%| ABOUT CALLBACKS:
%| GUIDE automatically appends subfunction prototypes to this file, and 
%| sets objects' callback properties to call them through the FEVAL 
%| switchyard above. This comment describes that mechanism.
%|
%| Each callback subfunction declaration has the following form:
%| <SUBFUNCTION_NAME>(H, EVENTDATA, HANDLES, VARARGIN)
%|
%| The subfunction name is composed using the object's Tag and the 
%| callback type separated by '_', e.g. 'slider2_Callback',
%| 'figure1_CloseRequestFcn', 'axis1_ButtondownFcn'.
%|
%| H is the callback object's handle (obtained using GCBO).
%|
%| EVENTDATA is empty, but reserved for future use.
%|
%| HANDLES is a structure containing handles of components in GUI using
%| tags as fieldnames, e.g. handles.figure1, handles.slider2. This
%| structure is created at GUI startup using GUIHANDLES and stored in
%| the figure's application data using GUIDATA. A copy of the structure
%| is passed to each callback.  You can store additional information in
%| this structure at GUI startup, and you can change the structure
%| during callbacks.  Call guidata(h, handles) after changing your
%| copy to replace the stored original so that subsequent callbacks see
%| the updates. Type "help guihandles" and "help guidata" for more
%| information.
%|
%| VARARGIN contains any extra arguments you have passed to the
%| callback. Specify the extra arguments by editing the callback
%| property in the inspector. By default, GUIDE sets the property to:
%| <MFILENAME>('<SUBFUNCTION_NAME>', gcbo, [], guidata(gcbo))
%| Add any extra arguments after the last argument, before the final
%| closing parenthesis.



% --------------------------------------------------------------------
function varargout = simulate_Callback(h, eventdata, handles, varargin)
%reset distances:
set(handles.maxPath,'string','0')
set(handles.meanPath,'string','0')
set(handles.minObstacle,'string','inf')
set(handles.timer,'String','0');

set(handles.abort,'Visible','on') %show abort button



% detect path following algorithm
val = get(handles.path_tracking,'value');
string_list = get(handles.path_tracking,'string');
pathFollowing  = string_list{val}; 

% detect obstacle avoidance algorithm
val = get(handles.obstacle_avoidance,'value');
string_list = get(handles.obstacle_avoidance,'string');
obstacleAvoidance  = string_list{val}; 
if(strcmp(obstacleAvoidance, 'No Obstacles'))
    noObstacles = 1;
    %obstacleAvoidance = 'VFH';
else
    noObstacles = 0;
end

v = str2num(get(handles.velocity,'string'));
dt = str2num(get(handles.dt,'string'));
variance = str2num(get(handles.noice,'string'))^2;
pathNr = get(handles.path,'value');
TC = get(handles.turningCircle,'value');
cla
%newplot
%start simulation
status = traktor(noObstacles, pathFollowing, obstacleAvoidance, v, dt, pathNr, handles, TC, 'sim', variance);
if(strcmp(status, 'goal'))
    %    msgbox('The vehicle has reached its destination', 'GOAL', 'warn');
elseif(strcmp(status,'error'))
    errordlg('The vehicle could not reach its destination', 'ERROR', 'on')
end

% --------------------------------------------------------------------
function varargout = velocity_Callback(h, eventdata, handles, varargin)
v = str2num(get(handles.velocity,'string'));
if isempty(v) | isreal(v)==0 
    errordlg('The velocity must be a number', 'Error', 'on');
    set(handles.velocity,'string', '0.5');
end

% --------------------------------------------------------------------
function varargout = dt_Callback(h, eventdata, handles, varargin)
dt = str2num(get(handles.dt,'string'));
if isempty(dt)| isreal(dt)==0
    errordlg('The time must be a number', 'Error', 'on');
    set(handles.dt,'string', '1');
end

% --------------------------------------------------------------------
function varargout = noice_Callback(h, eventdata, handles, varargin)
variance = str2num(get(handles.noice,'string'))^2;
if isempty(variance) | isreal(variance)==0
    errordlg('The noice must be a number', 'Error', 'on');
    set(handles.noice,'string', '0');
end


% --------------------------------------------------------------------
function varargout = path_Callback(h, eventdata, handles, varargin)
cla
thePath = selectPath(get(h,'val'));
drawPath([thePath.x thePath.y]);
obstacle_avoidance_Callback(handles.obstacle_avoidance, [], handles, varargin) %draw the obstacles

% --------------------------------------------------------------------
function varargout = obstacle_avoidance_Callback(h, eventdata, handles, varargin)
val = get(handles.obstacle_avoidance,'value');
string_list = get(handles.obstacle_avoidance,'string');
obstacleAvoidance = string_list{val}; % convert from cell array to string

obstacles = selectObstacles(get(handles.path,'value'));

if(~ strcmp(obstacleAvoidance, 'No Obstacles'))
    drawObstacles(obstacles); 
else
    cla
    thePath = selectPath(get(handles.path,'val'));
    drawPath([thePath.x thePath.y]);
end

% --------------------------------------------------------------------
function varargout = step_Callback(h, eventdata, handles, varargin)
uiresume




% --------------------------------------------------------------------
function varargout = pauseCheck_Callback(h, eventdata, handles, varargin)
pse = get(handles.pauseCheck,'value');
if(pse == 1)
    set(handles.step,'Visible','on')
    uiwait
else
    set(handles.step,'Visible','off')
end


% --------------------------------------------------------------------
function varargout = maxPath_Callback(h, eventdata, handles, varargin)
mp = str2num(get(handles.maxPath,'String'));
set(handles.maxPath,'String', num2str(max(mp, varargin{1}),'%0.2g'));


% --------------------------------------------------------------------
function varargout = meanPath_Callback(h, eventdata, handles, varargin)
mp = str2num(get(handles.meanPath,'String'));
set(handles.meanPath,'String', num2str(mean(varargin{1}),'%0.2g'));

% --------------------------------------------------------------------
function varargout = minObstacle_Callback(h, eventdata, handles, varargin)
mo = str2num(get(handles.minObstacle,'String'));
set(handles.minObstacle,'String', num2str(min(mo, varargin{1}),'%0.2g'));


% --------------------------------------------------------------------
function varargout = GUI_KeyPressFcn(h, eventdata, handles, varargin)
uiresume




% --------------------------------------------------------------------
function varargout = histogram_Callback(h, eventdata, handles, varargin)
varargout{1} = get(handles.histogram,'value');



% --------------------------------------------------------------------
function varargout = turningCircle_Callback(h, eventdata, handles, varargin)
varargout{1} = get(handles.turningCircle,'value');


% --------------------------------------------------------------------
function varargout = timer_Callback(h, eventdata, handles, varargin)
t = str2num(get(handles.timer,'String'));
dt = varargin{1}; %adds dt seconds for each call
set(handles.timer,'String', num2str(t+dt));





% --------------------------------------------------------------------
function varargout = abort_Callback(h, eventdata, handles, varargin)
uiresume
set(handles.abort,'Visible','off');
set(handles.abort,'Value',1);
gui;

% --------------------------------------------------------------------
function varargout = Help_Callback(h, eventdata, handles, varargin)
helpwin({'Velocity' 'The vehicles velocity in m/s.';...
        ...
        'Delta Time' 'The length of each timestep in seconds.';...
        ...
        'Noice' [{'Adds a noramlly distributed noice to the vehicles position.'},...
        {'The setting is for the standard deviation of the noice in meters.'}...
        {'Set to zero to disable the noice.'}];...
        ...
        'Path number' [{'Select the path that the vehicle will follow.'},...
            {'The corresponding path will be shown in the graph window.'}];...
        ...
        'Path Tracking Algorithm.' 'Select the desired method of path following.';...
        ...
        'Obstacle Avoidance Algorithm.' [{'Select the desired method of obstacle avoidance.'},...
            {'If "No Obstacles" is selected, there will be no obstacles on the path.'}];...
        ...
        'Plot Histograms' [{'Check to plot the Polar Histograms in the VFH-methods.'},...
            {'The histograms will open in a new window.'}];...
        ...
        'Plot the turning circle' [{'Check to plot the turning circle.'},...
            {'(The circle the vehicle whould be following if the turning angle were constant).'}...
            {'The setting takes effect when starting a new simulation.'}];...
        ...
        'Pause' [{'Check to pause the simulation.'},...
            {'While checked the "Next Timestep" button or any key pressed will step'},...
            {'through the simulation Delta Time seconds at a time.'},...
            {'To continue the simulation without pauses, uncheck the checkbox.'}];...
        ...
        'Start Simulation' 'Starts a new siumlation with the settings from above.';...
        ...
        'Next Timestep' 'Step one timestep (Delta Time seconds) if "Pause" is checked.';...
        ...
        'Timer' 'Time elapsed in soconds since start of the path in vehicle time.'},...
    ...
    'Velocity', 'Help for the Intelligent Forwarder Simulator','Interpreter', 'tex')
% --------------------------------------------------------------------
function varargout = About_Callback(h, eventdata, handles, varargin)
helpdlg([{'Created by Ola Ringdahl, ringdahl@cs.umu.se,'},...
        {'as a part of the IFOR-project at Ume University, Sweden.'}...
        {'http://www.cs.umu.se/research/IFORnav/navigation/'}...
        {''},{'Version 2.0, 2003:'},...
        {'- Added the "Follow The Past" obstacle avoidance algorithm '},...
        {'- More natural steering behaviour of the vehicle'},...
        {'- Added noice to the vehicles positioning'}],...
    ...
    'About Intelligent Forwarder Simulator');








