% Plots the histograms from the vfh-methods
%
% laSektor: corresponding sector to look ahead angle
% sect: the sektor corresponding to the chosen direction
% h: the histogram
% threshold: the limit for a valley
% orientation: The current orientation of the vehicle
% plottitle: The title above the graf
%
function plotHistogram(laSektor, sect, h, threshold, orientation, plotTitle)
if(isempty(sect))
    sect = 0; % plot even when error...
end
n=length(h);
alpha=360/n;
X = [1:n]* alpha; % Values on the x-axis (in degrees)
Y = [0:max(h)];
if(max(h) <=1)
    Y=[0:0.01:1]; %to get a smoother line
end
figure(2)
hold off
(bar(X,h));
hold on
for(i=1:length(threshold))
    line([1,max(X)],[threshold(i),threshold(i)], 'LineStyle','-.');    
end
title(plotTitle)
ylabel('Value')
xlabel('Angle (degrees)')
laAngle = laSektor*alpha;
goalAngle = sect*alpha;
orient = rad2deg(orientation);

line([laAngle laAngle], [0, max(h)*1.1], 'LineStyle','-', 'color','r'); %look ahead angle
line([goalAngle goalAngle], [0, max(h)*1.1], 'LineStyle','--', 'color','g'); % selected goal angle
line([orient orient], [0, max(h)*1.1], 'LineStyle',':', 'color','b'); % current orientation of the vehicle

 if(length(threshold) == 2)
     legend('Upper Threshold', 'Lower Threshold' ,'Look Ahead Angle','Goal Angle','Orientaion')
 elseif(length(threshold) == 1)
     legend('Threshold','Look Ahead Angle','Goal Angle','Orientaion')
 else
     legend('Look Ahead Angle','Goal Angle','Orientaion')
 end

figure(1)

