% calculates the distance between a point and a line
%
% algorithm from:
% http://geometryalgorithms.com/Archive/algorithm_0102/algorithm_0102.htm
%
% P: The coordinates of the Vehicle Center Point (VCP) [x,y]
% P0: The first end point of the path segment [x,y]
% P1: The second end point of the path segment [x,y]
%
% Returns:
% dist: distance between VCP and nearest path point
% segment: 1 if the VCP is between P0 and P1, else 0
% point: nearest point on the segment
%
function  [dist, segment, point] = pointLineDist(P, P0, P1)
v = P1 - P0; % the segment 
w = P - P0;  % a vector between the VCP and the segment
c1 = w*v';
c2 = v*v';
if(c1 <= 0 ) % the point is left of P0
    dist =  norm(P - P0);
    point = P0;
    segment = 0;   
elseif(c2 <= c1) % the point is right of P1
    dist =  norm(P - P1);
    point = P1;
    segment = 0;
else
    b = c1 / c2;
    Pb = P0 + b.*v;
    dist = norm(P - Pb);
    point = Pb; 
    segment = 1;
end


