% status = traktor(noObstacles, pathFollowing, obstacleAvoidance, v, dt, pathNr, handles, TC, simMode)
% 
% Simulates a forest machine with path tracking and obstacle avoidance
% 
% Author: Ola Ringdahl
% Mail: ringdahl@cs.umu.se
%
% noObstacles: 1 if no obstacle avoidance, else 0
% pathFollowing: 'Follow The Carrot', 'Pure Pursuit' or 'Follow The Past'.
% ostacleAvoidance: 'VFH' or 'VFH+'
% v: velocity (m/s)
% dt: Delta Time(s)
% pathNr: Wich path will be used
% handles: handles to the GUI components
% TC: 1 if the turning circle will be ploted, else 0
% simMode: 'sim' if simultion, 'mod' if model, 'real' if real vehicle
%			 
% Returns:
% status: 'goal' when the vehicle reached the goal, 'error' if the vehicle must stop
%
function status = traktor(noObstacles, pathFollowing, obstacleAvoidance, v, dt, pathNr, handles, TC, simMode, variance)
X=[]; Y=[]; stop=0; PD=[]; 
% V=[]; OR=[]; TRN=[]; %needed to create new paths
global thePath vehicle 
init = 1; rec=0;
thePath = selectPath(pathNr); % defines the path
randn('state',0);
maxGoalDist = 1.5;
newAngle = 0; %the steerring angle (radians)
if(strcmp(simMode,'sim')) % call the simulator
    if(~strcmp(obstacleAvoidance, 'No Obstacles'))
        global realObstacles 
        realObstacles = selectObstacles(pathNr); 
        drawObstacles(realObstacles)
        maxGoalDist = 2.5;
    end
    simTurn(newAngle, v, dt, TC, init, rec); %move the vehicle
elseif(strcmp(simMode,'mod'))
    % turnPioneer(newAngle, v, init...)
elseif(strcmp(simMode,'real'))
    % turnReal(newAngle, v, init...)
end
init = 0;
drawPath([thePath.x thePath.y])
w=1;
period=20;
while(w)
    mean = sin(2*pi/period)*sqrt(variance); % mean value of the noice
    vehicle.rand = mean + variance * randn(1,2); % simulate noice in the position 
    gui('timer_Callback',0,[],handles, dt);
    if(strcmp(pathFollowing, 'Follow The Carrot'))
        [newAngle,  stop, laplot, pathDist] =...
            followTheCarrot(obstacleAvoidance, handles, simMode);
    elseif(strcmp(pathFollowing, 'Pure Pursuit'))
        [newAngle, stop, laplot, pathDist] =...
            purePursuit(obstacleAvoidance, handles, simMode);
    elseif(strcmp(pathFollowing, 'Follow The Past'))
        [newAngle, stop, laplot, pathDist]  =... 
            followThePast(obstacleAvoidance, handles, simMode);
    end   
    PD = [PD, pathDist];
    gui('maxPath_Callback',0,[],handles, pathDist);
    gui('meanPath_Callback',0,[],handles, PD);
    if(stop==1)
        disp('E R R O R ! ! !')
        status = 'error';
        break % stop and signal error 
    end
    X = [X vehicle.x]; Y = [Y vehicle.y]; 
    if(strcmp(simMode,'sim'))
        simTurn(newAngle, v, dt, TC, init, rec);
    elseif(strcmp(simMode,'mod'))
        % turnPioneer(newAngle, v, init...)
    elseif(strcmp(simMode,'real'))
        % turnReal(newAngle, v, init...)
    end
    plot(vehicle.x, vehicle.y, '.b') % plot the path taken by the vehicle
   % if(norm([vehicle.x,vehicle.y] - [thePath.x(length(thePath.x)), thePath.y(length(thePath.x))]) < maxGoalDist)
   [rw, pos, dst] = getPathPoint([vehicle.x, vehicle.y], [thePath.x, thePath.y]);
   if(rw == length(thePath.x))
        disp('G O A L ! !!')
        status = 'goal';
        break % stop at goal	 
    end    
    gui('pauseCheck_Callback',0,[],handles);
    delete(laplot) 
end 
figure(1);
X = [X vehicle.x]; Y = [Y vehicle.y];
% plot(X, Y, '.b')  % plot the path taken by the vehicle
