% [angle, stop, noObstacles] = vfh(laPoint, handles, simMode)
% 
% Makes a vehicle avoid obstacles
%
% laPoint: the goal point [x,y]
% handles: handles to the GUI components
% simMode: 'sim' if simultion, 'mod' if model, 'real' if real vehicle
%
% Returns:
% angle: the angle of the best direction of travel in radians
% stop: 1 if the vehicle gets stuck, else 0
% noObstacles: 1 if there's no obstacles towards the carrot point, else 0
%
function [angle, stop, noObstacles] = vfh(laPoint, handles, simMode)
persistent prevAngle;
if isempty(prevAngle)
    prevAngle = 0;
end
global vehicle
if(strcmp(simMode,'sim'))
%    currPos = [vehicle.x vehicle.y];
    currPos = [vehicle.x vehicle.y] + vehicle.rand; % add noice to the position
    global realObstacles 
    if(rand(1) == 0 & rand(2) ==0)
        obstacles == realObstacles; % just to speed things up a bit :)
    else
        obstacles = [realObstacles(:,1) + vehicle.rand(1), realObstacles(:,2) + vehicle.rand(2)]; % add noice to the positions    
    end
else
%     obstacles = sensorSweep() %scan for obstacles if not a simulation
% currPos = [vehicle.x vehicle.y];
end

dmax = norm(laPoint - currPos); %distance between the vehicle and the carrot point
leftSektor = NaN; rightSektor = NaN;
alpha = deg2rad(5); n=72; %divide the obstacles in 72 sectors at 5 degres
stop=0; mag=zeros(1,n); k=[]; noObstacles=0;h = zeros(1,n);g=[];
smax = 14; %narrow or wide valley
threshold=43; %limit for a valley 
b=2; a=1+b*dmax; % constants 
C = 15; %certainty value
laSektor = laSekt(currPos, laPoint); %corresponding sector to carrot point

for(i=1:length(obstacles))
    d = norm(obstacles(i,:) - currPos); % distance between VCP and an obstacle
    do = d;
    for(j=1:8)
        do = min(norm(vehicle.corners(j,:)-realObstacles(i,:)),do); %distance from corners to obstacle
    end
    gui('minObstacle_Callback',0,[],handles, do); % min distance between vehicle and obstacles
    if(d <= dmax)
        dy = obstacles(i,2) - currPos(2);
        dx = obstacles(i,1) - currPos(1);
        beta = atan(dy/dx); %the angle between the vehicle and an obstacle
        if(dx < 0)
            if(dy > 0)
                beta = pi - abs(beta);
            else
                beta = pi + abs(beta);
            end
        else
            if(dy < 0)
                beta = 2*pi - abs(beta);
            end
        end
        kt = round(beta/alpha);
        if(kt == 0)
            kt = n;
        end
        mag(kt) = max(mag(kt), C^2*(a-b*d)); %magnitude 
    end
end
h=mag; %polar ostacle density (POD)
hs=hSmooth(h,n); % smoothed polar ostacle density (POD)
j=1;
k=1;
while(k<=n)
    if(hs(k) < threshold)
        kr = k; % right border of a valley
        while(k <= n & hs(k) < threshold)
            kl = k; % left border of a valley 
            k=k+1;
        end
        % calculate candidate directions:
        if(kl-kr > smax) % wide valley
            c(j)   = round(kl - smax/2); % towards the left side
            c(j+1) = round(kr + smax/2); % towards the right side
            j=j+2;
            if(laSektor >= kr & laSektor <= kl)
                c(j) = laSektor; % straight at look ahead
                j=j+1;
            end		
        elseif(kl-kr > smax/5) % narrow valley 
            c(j) = round((kr+kl)/2);
            j=j+1;
        end
    else
        k=k+1;
    end
end
my1=5; my2=2; my3=2; % cost constants
for(i=1:j-1)
    g(i) = my1 * dSekt(c(i),laSektor) + my2*dSekt(c(i),vehicle.or/alpha) + my3*dSekt(c(i), prevAngle/alpha); %cost function    
end
[v,i] = min(g); % Select the valley nerest to look ahead sector
angle = c(i)*alpha;
prevAngle = angle;
if(isempty(g))
    stop = 1; % All directions blocked!
    errordlg('The vehicle could not reach its destination', 'ERROR', 'on')
elseif(c(i) == laSektor)
    noObstacles = 1;
end
% global BO % TEST!
% BO = plot(obstacles(:,1),obstacles(:,2),'.r'); %TEST!!

if(gui('histogram_Callback',0,[],handles) == 1)
    plotHistogram(laSektor, c(i), h, [threshold], vehicle.or, 'Polar Histogram' );
    gui('pauseCheck_Callback',0,[],handles);
    plotHistogram(laSektor, c(i), hs, [threshold], vehicle.or, 'Smoothed Polar Histogram' );
    gui('pauseCheck_Callback',0,[],handles);
end


