% [angle, stop, noObstacles] = vfhPlus(laPoint, handles, simMode)
% 
% Makes a vehicle avoid obstacles
%
% laPoint: the goal point [x,y]
% handles: handles to the GUI components
% simMode: 'sim' if simultion, 'mod' if model, 'real' if real vehicle
%
% Returns:
% angle: the angle of the best direction of travel in radians
% stop: 1 if the vehicle has encountered problems and must stop, else 0
% noObstacles: 1 if there's no obstacles towards the carrot point, else 0
%
function [angle, stop, noObstacles] = vfhPlus(laPoint, handles, simMode)
global vehicle obstacles
if(strcmp(simMode,'sim'))
    currPos = [vehicle.x vehicle.y] + vehicle.rand; % add noice to the position
    global realObstacles 
    if(rand(1) == 0 & rand(2) ==0)
        obstacles == realObstacles; % just to speed things up a bit :)
    else
        obstacles = [realObstacles(:,1) + vehicle.rand(1), realObstacles(:,2) + vehicle.rand(2)]; % add noice to the positions    
    end
else
    % obstacles = sensorSweep() %scan for obstacles if not a simulation
    % currPos = [vehicle.x vehicle.y];
end
persistent prevAngle
if isempty(prevAngle)
    prevAngle = 0;
end
alpha = deg2rad(5); n=72; % divide the obstacles in 72 sectors at 5 degres
leftSektor = NaN; rightSektor = NaN;stop=0; 
hp=zeros(1,n);hb=zeros(1,n);hPrim=zeros(1,n);
mag=zeros(1,n);c=[];g=[]; noObstacles=0;
dmax = 3.5*vehicle.length; 
minDist = 0.2*vehicle.length; % the minimum distance to an obstacle
rLarge = vehicle.length;% + minDist; %r_r+s
smax =8;  % narrow or wide valley
tlow=3000; % lower limit for a valley 
thigh=5000; % upper limit for a valley
b= 2; a=1+b*dmax^2; % used to calculate mag 
C = 15; % certainty value
laSektor = laSekt(currPos, laPoint); % corresponding sector to lookAhead 

%position of left & right trajectory centers:
[TClx,TCly] = pol2cart(vehicle.or+pi/2, vehicle.minRadius); 	
TCl = currPos + [TClx,TCly];
[TCrx,TCry] = pol2cart(vehicle.or-pi/2, vehicle.minRadius); 
TCr = currPos + [TCrx,TCry];

% left & right limit angles:
phib = mod(vehicle.or+pi, 2*pi);
phir = phib;
phil = phib;
[nrOfObstacles,col] = size(obstacles);
for(i=1:nrOfObstacles)
    d = norm(obstacles(i,:) - currPos); % distance between the vehicle and an obstacle
    do = d;
    for(j=1:8)
        do = min(norm(vehicle.corners(j,:)-realObstacles(i,:)),do); %distance from corners to obstacle
    end
    gui('minObstacle_Callback',0,[],handles, do); % min distance between vehicle and obstacles
    if(d <= dmax)
        dy = obstacles(i,2) - currPos(2);
        dx = obstacles(i,1) - currPos(1);
        beta = atan(dy/dx); % angle between the vehicle and an obstacle
        if(dx < 0)
            if(dy > 0)
                beta = pi - abs(beta);
            else
                beta = pi + abs(beta);
            end
        elseif(dy < 0)
            beta = 2*pi - abs(beta);
        end
        k = round(beta/alpha); % corresponding sector to the obstacle
        if(k == 0)
            k = n;
        end
        mag(k) = max(mag(k), C^2*(a-b*d^2)); %magnitude
        gamma = asin(rLarge/d);
        if(~ isreal(gamma))
            gamma = asin(1); % if rLarge > d
        end
        for(j=k+1:n)
            hPrim(j) = hprim(j,gamma,beta, hPrim(j)); %used in primary polar histogram 
            if(hPrim(j) == 0)
                break
            end
            mag(j) =max(mag(j),mag(k));
        end
        for(j=k:-1:1)
            hPrim(j) = hprim(j,gamma,beta, hPrim(j)); %used in primary polar histogram 
            if(hPrim(j) == 0)
                break
            end
            mag(j) = max(mag(j),mag(k));
        end
        % distance from obstacle to the trajectory centers:
        dr = norm(TCr-obstacles(i,:));
        dl = norm(TCl-obstacles(i,:));
        betaor = compAngles(beta,vehicle.or);
        if(strcmp(betaor,'r') & strcmp(compAngles(beta,phir),'l'))
            if(dr < (vehicle.minRadius+rLarge)) % obstacle blocks the directions to its right
                phir = beta;
            end
        elseif(strcmp(betaor,'l') & strcmp(compAngles(beta,phil),'r'))
            if(dl < (vehicle.minRadius+rLarge)) % obstacle blocks the directions to its left
                phil = beta;
            end
        end	  
    end
end
for(i=1:n)
    hp(i) = mag(i)*hPrim(i); %primary polar histogram (hp)
    %binary polar histogram (hb):
    if(hp(i) > thigh)
        hb(i) = 1;
    elseif(hp(i) < tlow)
        hb(i) = 0;
    elseif(i>1)
        hb(i) = hb(i-1);
    end
    %masked polar histogram (hm):
    philSect = compAngles(i*alpha, phil);
    phirSect = compAngles(i*alpha, phir);
    orSect	 = compAngles(i*alpha, vehicle.or);
    
    if(hb(i)==0 & strcmp(philSect,'r') &  strcmp(orSect,'l'))
        hm(i) = 0; %between phil & orientation
    elseif(hb(i)==0 & strcmp(phirSect,'l') &  strcmp(orSect,'r'))
        hm(i) = 0; %between phir & orientation
    elseif(hb(i)==0 & phil == phir & phir == phib)
        hm(i) = 0; %between phir & orientation
    else
        hm(i) = 1;	
    end
end
% detect candidate valleys:
j=1;
k=1;
while(k<=n)
    if(hm(k) == 0)
        kr = k; % right border of a valley
        while(k <= n & hm(k) == 0)
            kl = k; % left border of a valley 
            k=k+1;
        end
        % calculate candidate directions:
        if(kl-kr > smax) % wide valley
            c(j) = round(kl - smax/2); % towards the left side
            c(j+1) = round(kr + smax/2); % towards the right side
            j=j+2;
            if(laSektor >= kr & laSektor <= kl)
                c(j) = laSektor; % straight at look ahead
                j=j+1;
            end		
        else % narrow valley 
            c(j) = round((kr+kl)/2);
            j=j+1;
        end
    else
        k=k+1;
    end
end
%cost funnction:
my1=5; my2=2; my3=2; % cost constants
for(i=1:j-1)
    g(i) = my1*dSekt(c(i),laSektor) + my2*dSekt(c(i),vehicle.or/alpha) +...
        my3*dSekt(c(i), prevAngle/alpha);
end
[v,i] = min(g);
angle = c(i)*alpha;
prevAngle = angle;
if(isempty(g))
    stop = 1; % All directions blocked!
    errordlg('The vehicle could not reach its destination', 'ERROR', 'on')
elseif(c(i) == laSektor)
    noObstacles = 1;
end
if(gui('histogram_Callback',0,[],handles) == 1)
    plotHistogram(laSektor, c(i), hp, [tlow,thigh], vehicle.or, 'Primary Polar Histogram');
    gui('pauseCheck_Callback',0,[],handles);
    plotHistogram(laSektor, c(i), hb, [],vehicle.or, 'Binary Polar Histogram');
    gui('pauseCheck_Callback',0,[],handles);
    plotHistogram(laSektor, c(i), hm, [],vehicle.or, 'Masked Polar Histogram');
end
